<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */
namespace TestIntegration\Model;

use Application\Config\ConfigException;
use Application\Config\ConfigManager;
use Application\Config\IConfigDefinition;
use Application\Log\SwarmLogger;
use Interop\Container\ContainerInterface;
use P4\Connection\ConnectionInterface;
use TestIntegration\Filter\ITestDefinition as Filter;

/**
 * Class TestDefinitionDAO to fetch/build/save TestDefinition
 * @package TestIntegration\Model
 */
class TestDefinitionDAO implements ITestDefinitionDAO
{
    private $config;
    private $services;

    /**
     * @inheritDoc
     */
    public function __construct(ContainerInterface $services, array $options = null)
    {
        $this->services = $services;
        $this->config   = $services->get(IConfigDefinition::CONFIG);
    }

    /**
     * Fetch all test definitions with the option to limit by names
     * @param array                     $options        FETCH_BY_NAMES an array of names of test defintions to fetch
     * @param ConnectionInterface|null  $connection     unused at present as test definitions are held in config and
     *                                                  not Perforce (this may change in future)
     * @return array valid test definitions with invalid ones excluded
     * @throws ConfigException  if the overall config cannot be parsed
     */
    public function fetchAll(array $options = [], ConnectionInterface $connection = null)
    {
        $options = $options + [
            self::FETCH_BY_NAMES => []
        ];

        $configuredTests = ConfigManager::getValue($this->config, ConfigManager::GLOBAL_TESTS, []);
        $testDefinitions = [];
        $errors          = [];
        foreach ($configuredTests as $configuredTest) {
            $filter = $this->services->get(Filter::NAME);
            $filter->setData($configuredTest);
            if ($filter->isValid()) {
                if (!$options[self::FETCH_BY_NAMES] ||
                    (
                        is_array($options[self::FETCH_BY_NAMES])
                        && in_array(
                            $configuredTest[TestDefinition::FIELD_NAME], $options[self::FETCH_BY_NAMES]
                        )
                    )
                ) {
                    $td = new TestDefinition;
                    $td->set($configuredTest);
                    $testDefinitions[] = $td;
                }
            } else {
                // encode the messages as a convenient way to handle deep array filter messages
                $errors[] = sprintf(
                    "name [%s], errors [%s]",
                    $configuredTest[TestDefinition::FIELD_NAME],
                    json_encode($filter->getMessages())
                );
            }
        }
        if ($errors) {
            $logger = $this->services->get(SwarmLogger::SERVICE);
            foreach ($errors as $encodedError) {
                $logger->err(
                    sprintf(
                        "[%s]: Error in global test definition with %s",
                        get_class($this),
                        $encodedError
                    )
                );
            }
        }
        return $testDefinitions;
    }
}
